@@ -188,6 +188,14 @@ class IntArg(ArgType):
         info.varlist.add('int', 'ret')
         info.codeafter.append('    return PyInt_FromLong(ret);')
 
+class BoolArg(IntArg):
+    def write_return(self, ptype, ownsreturn, info):
+        info.varlist.add('int', 'ret')
+        info.varlist.add('PyObject', '*py_ret')
+        info.codeafter.append('    py_ret = ret ? Py_True : Py_False;\n'
+                              '    Py_INCREF(py_ret);\n'
+                              '    return py_ret;')
+
 class TimeTArg(ArgType):
     def write_param(self, ptype, pname, pdflt, pnull, info):
 	if pdflt:
@@ -396,7 +404,7 @@ class ObjectArg(ArgType):
 		info.codebefore.append(self.dflt % {'name':pname,
                                                     'cast':self.cast}) 
 		info.arglist.append(pname)
-                info.add_parselist('O', ['&Py%s_Type' % self.objname,
+                info.add_parselist('O!', ['&Py%s_Type' % self.objname,
                                          '&py_' + pname], [pname])
 	    else:
 		info.varlist.add('PyGObject', '*' + pname)
@@ -547,14 +555,24 @@ class PointerArg(ArgType):
                                   '    return pyg_pointer_new(' + self.typecode + ', &ret);')
 
 class AtomArg(IntArg):
+    dflt = '    if (py_%(name)s) {\n' \
+           '        %(name)s = pygdk_atom_from_pyobject(py_%(name)s);\n' \
+           '        if (PyErr_Occurred())\n' \
+           '            return NULL;\n' \
+           '    }\n'
     atom = ('    %(name)s = pygdk_atom_from_pyobject(py_%(name)s);\n'
             '    if (PyErr_Occurred())\n'
             '        return NULL;\n')
     def write_param(self, ptype, pname, pdflt, pnull, info):
-        info.varlist.add('GdkAtom', pname)
-	info.varlist.add('PyObject', '*py_' + pname + ' = NULL')
-	info.codebefore.append(self.atom % {'name': pname})
-	info.arglist.append(pname)
+        if pdflt:
+            info.varlist.add('GdkAtom', pname + ' = ' + pdflt)
+            info.varlist.add('PyObject', '*py_' + pname + ' = NULL')
+            info.codebefore.append(self.dflt % {'name': pname})
+        else:
+            info.varlist.add('GdkAtom', pname)
+            info.varlist.add('PyObject', '*py_' + pname + ' = NULL')
+            info.codebefore.append(self.atom % {'name': pname})
+        info.arglist.append(pname)
         info.add_parselist('O', ['&py_' + pname], [pname])
     def write_return(self, ptype, ownsreturn, info):
         info.varlist.add('GdkAtom', 'ret')
@@ -596,12 +614,6 @@ class GtkTreePathArg(ArgType):
             '            return NULL;\n'
             '        }\n'
             '    }\n')
-    null = ('    if (PyTuple_Check(py_%(name)s))\n'
-            '        %(name)s = pygtk_tree_path_from_pyobject(py_%(name)s);\n'
-            '    else if (py_%(name)s != Py_None) {\n'
-            '        PyErr_SetString(PyExc_TypeError, "%(name)s should be a GtkTreePath or None");\n'
-            '        return NULL;\n'
-            '    }\n')
     freepath = ('    if (%(name)s)\n'
                 '        gtk_tree_path_free(%(name)s);\n')
     def __init__(self):
@@ -639,20 +651,52 @@ class GtkTreePathArg(ArgType):
                                   '    return Py_None;')
 						       
 class GdkRectanglePtrArg(ArgType):
-    null = ('    if (!pygdk_rectangle_from_pyobject(py_%(name)s, &%(name)s))\n'
-            '        return NULL;\n')
+    normal = ('    if (!pygdk_rectangle_from_pyobject(py_%(name)s, &%(name)s))\n'
+              '        return NULL;\n')
+    null =   ('    if (py_%(name)s == Py_None)\n'
+              '        %(name)s = NULL;\n'
+              '    else if (pygdk_rectangle_from_pyobject(py_%(name)s, &%(name)s_rect))\n'
+              '        %(name)s = &%(name)s_rect;\n'
+              '    else\n'
+              '            return NULL;\n')
     def write_param(self, ptype, pname, pdflt, pnull, info):
-        info.varlist.add('GdkRectangle', pname + ' = { 0, 0, 0, 0 }')
-        info.varlist.add('PyObject', '*py_' + pname + ' = Py_None')
-        info.add_parselist('O', ['&py_' + pname], [pname])
-	info.arglist.append('&' + pname)
-	info.codebefore.append(self.null % {'name':  pname})
+        if pnull:
+            info.varlist.add('GdkRectangle', pname + '_rect = { 0, 0, 0, 0 }')
+            info.varlist.add('GdkRectangle', '*' + pname)
+            info.varlist.add('PyObject', '*py_' + pname + ' = Py_None')
+            info.add_parselist('O', ['&py_' + pname], [pname])
+            info.arglist.append(pname)
+            info.codebefore.append(self.null % {'name':  pname})
+        else:
+            info.varlist.add('GdkRectangle', pname + ' = { 0, 0, 0, 0 }')
+            info.varlist.add('PyObject', '*py_' + pname)
+            info.add_parselist('O', ['&py_' + pname], [pname])
+            info.arglist.append('&' + pname)
+            info.codebefore.append(self.normal % {'name':  pname})
 
 class GdkRectangleArg(ArgType):
     def write_return(self, ptype, ownsreturn, info):
 	info.varlist.add('GdkRectangle', 'ret')
 	info.codeafter.append('    return pyg_boxed_new(GDK_TYPE_RECTANGLE, &ret, TRUE, TRUE);')
 
+class PyObjectArg(ArgType):
+    def write_param(self, ptype, pname, pdflt, pnull, info):
+        info.varlist.add('PyObject', '*' + pname)
+        info.add_parselist('O', ['&' + pname], [pname])
+        info.arglist.append(pname)
+    def write_return(self, ptype, ownsreturn, info):
+        info.varlist.add("PyObject", "*ret")
+        if ownsreturn:
+            info.codeafter.append('    if (ret) {\n'
+                                  '       return ret;\n'
+                                  '    }\n'
+                                  '    Py_INCREF(Py_None);\n'
+                                  '    return Py_None;')
+        else:
+            info.codeafter.append('    if (!ret) ret = Py_None;\n'
+                                  '    Py_INCREF(ret);\n'
+                                  '    return ret;')
+
 class ArgMatcher:
     def __init__(self):
 	self.argtypes = {}
@@ -742,14 +786,17 @@ matcher.register('gshort', arg)
 matcher.register('gushort', arg)
 matcher.register('long', arg)
 matcher.register('glong', arg)
-matcher.register('gboolean', arg)
-
+matcher.register('gsize', arg)
+matcher.register('gssize', arg)
 matcher.register('guint8', arg)
 matcher.register('gint8', arg)
 matcher.register('guint16', arg)
 matcher.register('gint16', arg)
 matcher.register('gint32', arg)
 
+arg = BoolArg()
+matcher.register('gboolean', arg)
+
 arg = TimeTArg()
 matcher.register('time_t', arg)
 
@@ -793,6 +840,7 @@ matcher.register('GtkTreePath*', GtkTreePathArg())
 matcher.register('GdkRectangle*', GdkRectanglePtrArg())
 matcher.register('GtkAllocation*', GdkRectanglePtrArg())
 matcher.register('GdkRectangle', GdkRectangleArg())
+matcher.register('PyObject*', PyObjectArg())
 
 matcher.register('GdkNativeWindow', ULongArg())
 
