@@ -132,6 +132,10 @@ static gboolean pygtk_generic_tree_model_iter_nth_child(GtkTreeModel *tree_model
 static gboolean pygtk_generic_tree_model_iter_parent(GtkTreeModel *tree_model,
 					     GtkTreeIter *iter,
 					     GtkTreeIter *child);
+static void pygtk_generic_tree_model_unref_node(GtkTreeModel *tree_model,
+					     GtkTreeIter *iter);
+static void pygtk_generic_tree_model_ref_node(GtkTreeModel *tree_model,
+					     GtkTreeIter *iter);
 
 static void
 pygtk_generic_tree_model_iface_init(GtkTreeModelIface *iface)
@@ -148,6 +152,9 @@ pygtk_generic_tree_model_iface_init(GtkTreeModelIface *iface)
   iface->iter_n_children = pygtk_generic_tree_model_iter_n_children;
   iface->iter_nth_child = pygtk_generic_tree_model_iter_nth_child;
   iface->iter_parent = pygtk_generic_tree_model_iter_parent;
+  iface->ref_node = pygtk_generic_tree_model_ref_node;
+  iface->unref_node = pygtk_generic_tree_model_unref_node;
+
 }
 
 static void
@@ -342,20 +349,25 @@ pygtk_generic_tree_model_get_iter(GtkTreeModel *tree_model,
 static GtkTreePath *
 pygtk_generic_tree_model_get_path(GtkTreeModel *tree_model, GtkTreeIter *iter)
 {
-    PyObject *self, *py_ret;
+    PyObject *self, *py_ret, *py_iter;
 
     g_return_val_if_fail(tree_model != NULL, NULL);
     g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), NULL);
     g_return_val_if_fail(iter != NULL, NULL);
+
+    pyg_block_threads();
+
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
 
 #ifdef DEBUG_TREE_MODEL
     g_message("get_path(%p)", iter);
 #endif
-    pyg_block_threads();
-    py_ret = PyObject_CallMethod(self, METHOD_PREFIX "get_path",
-				 "(O)", (PyObject *)iter->user_data);
+    py_iter = (PyObject *)iter->user_data;
+    if (py_iter == NULL)
+	py_iter = Py_None;
+
+    py_ret = PyObject_CallMethod(self, METHOD_PREFIX "get_path", "(O)", py_iter);
     if (py_ret) {
 	GtkTreePath *path = pygtk_tree_path_from_pyobject(py_ret);
 
@@ -376,7 +388,7 @@ static void
 pygtk_generic_tree_model_get_value(GtkTreeModel*tree_model, GtkTreeIter *iter,
 			   gint column, GValue *value)
 {
-    PyObject *self, *py_value;
+    PyObject *self, *py_value, *py_iter;
 
     g_return_if_fail(tree_model != NULL);
     g_return_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model));
@@ -394,8 +406,11 @@ pygtk_generic_tree_model_get_value(GtkTreeModel*tree_model, GtkTreeIter *iter,
     /* init value to column type */
     g_value_init(value, pygtk_generic_tree_model_get_column_type(tree_model, column));
 
+    py_iter = (PyObject *)iter->user_data;
+    if (py_iter == NULL)
+	py_iter = Py_None;
     py_value = PyObject_CallMethod(self, METHOD_PREFIX "get_value",
-				   "(Oi)", (PyObject *)iter->user_data,column);
+				   "(Oi)", py_iter,column);
 
     if (py_value) {
 	pyg_value_from_pyobject(value, py_value);
@@ -409,7 +424,7 @@ pygtk_generic_tree_model_get_value(GtkTreeModel*tree_model, GtkTreeIter *iter,
 static gboolean
 pygtk_generic_tree_model_iter_next(GtkTreeModel *tree_model, GtkTreeIter *iter)
 {
-    PyObject *self, *py_ret;
+    PyObject *self, *py_ret, *py_iter;
 
     g_return_val_if_fail(tree_model != NULL, FALSE);
     g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), FALSE);
@@ -423,8 +438,11 @@ pygtk_generic_tree_model_iter_next(GtkTreeModel *tree_model, GtkTreeIter *iter)
 #ifdef DEBUG_TREE_MODEL
     g_message("iter_next(%p)", iter);
 #endif
-    py_ret = PyObject_CallMethod(self, METHOD_PREFIX "iter_next",
-				 "(O)", (PyObject *)iter->user_data);
+    py_iter = (PyObject *)iter->user_data;
+    if (py_iter == NULL)
+	py_iter = Py_None;
+
+    py_ret = PyObject_CallMethod(self, METHOD_PREFIX "iter_next", "(O)", py_iter);
     if (py_ret) {
 	if (py_ret != Py_None) {
 	    /* XXXX handle reference counting here */
@@ -496,7 +514,7 @@ pygtk_generic_tree_model_iter_children(GtkTreeModel *tree_model, GtkTreeIter *it
 static gboolean
 pygtk_generic_tree_model_iter_has_child(GtkTreeModel *tree_model, GtkTreeIter *iter)
 {
-    PyObject *self, *py_ret;
+    PyObject *self, *py_ret, *py_iter;
 
     g_return_val_if_fail(tree_model != NULL, FALSE);
     g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), FALSE);
@@ -510,8 +528,12 @@ pygtk_generic_tree_model_iter_has_child(GtkTreeModel *tree_model, GtkTreeIter *i
 #ifdef DEBUG_TREE_MODEL
     g_message("iter_has_child(%p)", iter);
 #endif
+    py_iter = (PyObject *)iter->user_data;
+    if (py_iter == NULL)
+	py_iter = Py_None;
+
     py_ret = PyObject_CallMethod(self, METHOD_PREFIX "iter_has_child",
-				 "(O)", (PyObject *)iter->user_data);
+				 "(O)", py_iter);
     if (py_ret) {
 	gboolean ret = PyObject_IsTrue(py_ret);
 
@@ -528,11 +550,10 @@ pygtk_generic_tree_model_iter_has_child(GtkTreeModel *tree_model, GtkTreeIter *i
 static gint
 pygtk_generic_tree_model_iter_n_children(GtkTreeModel *tree_model, GtkTreeIter *iter)
 {
-    PyObject *self, *py_ret;
+    PyObject *self, *py_ret, *py_iter;
 
     g_return_val_if_fail(tree_model != NULL, FALSE);
     g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), FALSE);
-    g_return_val_if_fail(iter != NULL, FALSE);
 
     pyg_block_threads();
 
@@ -542,8 +563,10 @@ pygtk_generic_tree_model_iter_n_children(GtkTreeModel *tree_model, GtkTreeIter *
 #ifdef DEBUG_TREE_MODEL
     g_message("iter_n_children(%p)", iter);
 #endif
+    py_iter = iter != NULL ? (PyObject *)iter->user_data : Py_None;
+
     py_ret = PyObject_CallMethod(self, METHOD_PREFIX "iter_n_children",
-				 "(O)", (PyObject *)iter->user_data);
+				 "(O)", py_iter);
     if (py_ret) {
 	gint ret = PyInt_AsLong(py_ret);
 
@@ -647,3 +670,74 @@ pygtk_generic_tree_model_iter_parent(GtkTreeModel *tree_model, GtkTreeIter *iter
     }
 }
 
+static void
+pygtk_generic_tree_model_unref_node(GtkTreeModel *tree_model, GtkTreeIter *iter)
+{
+    PyObject *self, *py_ret, *py_iter, *method;
+
+    g_return_if_fail(tree_model != NULL);
+    g_return_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model));
+    g_return_if_fail(iter != NULL);
+
+    pyg_block_threads();
+
+    /* this call finds the wrapper for this GObject */
+    self = pygobject_new((GObject *)tree_model);
+
+#ifdef DEBUG_TREE_MODEL
+    g_message("unref_node(%p)", iter);
+#endif
+    py_iter = (PyObject *)iter->user_data;
+    if (py_iter == NULL)
+	py_iter = Py_None;
+
+    method = PyObject_GetAttrString(self, METHOD_PREFIX "unref_node");
+    if (method == NULL)
+	PyErr_Clear();
+    else {
+	py_ret = PyObject_CallFunction(method, "(O)", py_iter);
+	if (py_ret) {
+	    Py_DECREF(py_ret);
+	} else {
+	    PyErr_Print();
+	}
+    }
+
+    pyg_unblock_threads();
+}
+
+static void
+pygtk_generic_tree_model_ref_node(GtkTreeModel *tree_model, GtkTreeIter *iter)
+{
+    PyObject *self, *py_ret, *py_iter, *method;
+
+    g_return_if_fail(tree_model != NULL);
+    g_return_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model));
+    g_return_if_fail(iter != NULL);
+
+    pyg_block_threads();
+
+    /* this call finds the wrapper for this GObject */
+    self = pygobject_new((GObject *)tree_model);
+
+#ifdef DEBUG_TREE_MODEL
+    g_message("ref_node(%p)", iter);
+#endif
+    py_iter = (PyObject *)iter->user_data;
+    if (py_iter == NULL)
+	py_iter = Py_None;
+
+    method = PyObject_GetAttrString(self, METHOD_PREFIX "ref_node");
+    if (method == NULL)
+	PyErr_Clear();
+    else {
+	py_ret = PyObject_CallFunction(method, "(O)", py_iter);
+	if (py_ret) {
+	    Py_DECREF(py_ret);
+	} else {
+	    PyErr_Print();
+	}
+    }
+
+    pyg_unblock_threads();
+}
